# -*- coding: utf-8 -*-
from __future__ import (division, print_function, unicode_literals)
#############################################################################
# Author  : Jerome ODIER, Jerome FULACHIER, Fabian LAMBERT, Solveig ALBRAND
#
# Email   : jerome.odier@lpsc.in2p3.fr
#           jerome.fulachier@lpsc.in2p3.fr
#           fabian.lambert@lpsc.in2p3.fr
#           solveig.albrand@lpsc.in2p3.fr
#
# Version : 5.X.X (2014)
#
#############################################################################

import pyAMI.utils, pyAMI.config, pyAMI.exception, pyAMI_atlas.utils

#############################################################################

def init():
	pyAMI.config.tables['projects'] = {
		'tag': 'projectTag',
		'description': 'description',
		'is_base_type': 'isBaseType',
		'read_status': 'readStatus=VALID',
		'write_status': 'writeStatus',
		'manager': 'projectManager',

		'@entity': 'projects',
		'@project': 'Atlas_Production',
		'@processing_step': '*',

		'@primary': 'tag',
		'@foreign': 'nomenclature',
	}

	pyAMI.config.tables['subprojects'] = {
		'tag': 'subProjectTag',
		'description': 'description',
		'is_base_type': 'isBaseType',
		'read_status': 'readStatus=VALID',
		'write_status': 'writeStatus',
		'manager': 'projectManager',

		'@entity': 'subprojects',
		'@project': 'Atlas_Production',
		'@processing_step': '*',

		'@primary': 'tag',
	}

	pyAMI.config.tables['types'] = {
		'name': 'dataType',
		'description': 'description',
		'read_status': 'readStatus=VALID',
		'write_status': 'writeStatus',

		'@entity': 'data_type',
		'@project': 'Atlas_Production',
		'@processing_step': '*',

		'@primary': 'name',
	}

	pyAMI.config.tables['subtypes'] = {
		'name': 'subDataType',
		'description': 'description',
		'read_status': 'readStatus=VALID',
		'write_status': 'writeStatus',

		'@entity': 'subData_type',
		'@project': 'Atlas_Production',
		'@processing_step': '*',

		'@primary': 'name',
	}

	pyAMI.config.tables['prodsteps'] = {
		'tag': 'productionStepTag',
		'name': 'productionStepName',
		'read_status': 'readStatus=VALID',
		'write_status': 'writeStatus',

		'@entity': 'productionStep',
		'@project': 'Atlas_Production',
		'@processing_step': '*',

		'@primary': 'tag, name',
	}

	pyAMI.config.tables['nomenclature'] = {
		'tag': 'nomenclatureTag',
		'template': 'nomenclatureTemplate',
		'description': 'nomenclatureName',
		'read_status': 'readStatus=VALID',
		'write_status': 'writeStatus',

		'@entity': 'nomenclature',
		'@project': 'Atlas_Production',
		'@processing_step': '*',

		'@primary': 'tag, template, description',
	}

	pyAMI.config.tables['datasets'] = {
		'ami_status': 'amiStatus=VALID',
		'atlas_release': 'AtlasRelease',
		'beam': 'beamType',
		'conditions_tag': 'conditionsTag',
		'cross_section': 'crossSection',
		'data_period': 'period',
		'dataset_number': 'datasetNumber',
		'ecm_energy': 'ecmEnergy',
		'events': 'totalEvents',
		'generator_name': 'generatorName',
		'generator_tune': 'generatorTune',
		'geometry': 'geometryVersion',
		'in_container': 'inContainer',
		'job_config': 'jobConfig',
		'ldn': 'logicalDatasetName',
		'modified': 'lastModified',
		'nfiles': 'nFiles',
		'pdf': 'PDF',
		'physics_comment': 'physicsComment',
		'physics_short': 'physicsShort',
		'production_step': 'productionStep',
		'prodsys_status': 'prodsysStatus',
		'project': 'projectName',
		'requested_by': 'requestedBy',
		'responsible': 'physicistResponsible',
		'run_number': 'runNumber',
		'stream': 'streamName',
		'total_size': 'totalSize',
		'transformation_package': 'TransformationPackage',
		'trash_annotation': 'trashAnnotation',
		'trash_date': 'trashDate',
		'trash_trigger': 'trashTrigger',
		'trigger_config': 'triggerConfig',
		'type': 'dataType',
		'version': 'version',

		'@entity': 'dataset',
		'@project': 'Atlas_Production',
		'@processing_step': 'Atlas_Production',

		'@primary': 'ldn',
		'@foreign': 'files',
	}

	pyAMI.config.tables['files'] = {
		'lfn': 'LFN',
		'guid': 'fileGUID',
		'size': 'fileSize',
		'events': 'events',
		'input_file': 'inputFile',
		'gen_filt_eff': 'GenFiltEff',
		'cross_section': 'crossSection',

		'@entity': 'files',
		'@project': 'Atlas_Production',
		'@processing_step': 'Atlas_Production',

		'@primary': 'lfn',
	}

#############################################################################
# AMI-TAGS                                                                  #
#############################################################################

def get_ami_tag(client, tag):
	'''Get AMI-tag information.

	Args:
	    :client: the pyAMI client [ pyAMI.client.Client ]
	    :tag: the AMI-tag [ str ]

	Returns:
	    a python dictionnaries.
	'''

	command = [
		'AMIGetAMITagInfo',
		'-amiTag="%s"' % tag,
	]

	return client.execute(command, format = 'dom_object').get_rows('amiTagInfo')

#############################################################################

def mode_ami_tag(client, args):
	pyAMI_atlas.utils.print_list(get_ami_tag(client, args['tag']), title = 'tag')

	return 0

#############################################################################
# DATASET INFO                                                              #
#############################################################################

def get_dataset_info(client, dataset):
	'''Get dataset information.

	Args:
	    :client: the pyAMI client [ pyAMI.client.Client ]
	    :dataset: the dataset [ str ]

	Returns:
	    a python dictionnaries.
	'''

	command = [
		'AMIGetDatasetInfo',
		'-logicalDatasetName="%s"' % dataset,
	]

	return client.execute(command, format = 'dict_object').get_rows()

#############################################################################

def mode_get_dataset_info(client, args):
	pyAMI_atlas.utils.print_list(get_dataset_info(client, args['dataset']))

	return 0

#############################################################################
# DATASET PROV                                                              #
#############################################################################

def get_dataset_prov(client, dataset):
	'''Get dataset provenance.

	Args:
	    :client: the pyAMI client [ pyAMI.client.Client ]
	    :dataset: the dataset [ str ]

	Returns:
	    a map of python dictionnaries. The key "node" gives a list of dataset with the distance to the given dataset and the key "edge" gives the list of successive pairs of input an output datasets.
	'''

	command = [
		'AMIGetDatasetProv',
		'-logicalDatasetName="%s"' % dataset,
	]

	result = client.execute(command, format = 'dict_object')

	return {'node': result.get_rows('node'),
	          'edge': result.get_rows('edge')}

#############################################################################

def mode_get_dataset_prov(client, args):
	provenance = get_dataset_prov(client, args['dataset'])

	pyAMI_atlas.utils.print_table(provenance['node'])
	pyAMI_atlas.utils.print_table(provenance['edge'])

	return 0

#############################################################################
# LIST COMMANDS                                                             #
#############################################################################

def list_commands(client):
	'''List AMI commands.

	Args:
	    :client: the pyAMI client [ pyAMI.client.Client ]

	Returns:
	    an array of python dictionnaries.
	'''

	command = [
		'AMIListCommands',
	]

	return client.execute(command, format = 'dict_object').get_rows()

#############################################################################

def mode_list_commands(client, args):
	pyAMI_atlas.utils.print_table(list_commands(client))

	return 0

#############################################################################
# LIST PROJECTS                                                             #
#############################################################################

def list_projects(client, patterns = None, fields = None, order = None, limit = None, show_archived = False, **kwargs):
	'''List ATLAS projects.

	Args:
	    :client: the pyAMI client [ pyAMI.client.Client ]
	    :patterns: the list of project patterns (glob with %) [ list<str> | None ]
	    :fields: the list of extra fields to be shown if the result [ list<str> | None ]
	    :order: the list of fields for ordering the result [ list<str> | None ]
	    :limit: limit number of results [ int | None ]
	    :show_archived: show archived results [ bool ]

	kwargs:
	    additionnal constraints field_name:field_value (glob with %)

	Returns:
	    an array of python dictionnaries.
	'''

	if not 'read_status' in kwargs:
		kwargs['read_status'] = 'VALID'

	return pyAMI.utils.smart_execute(client, 'projects', patterns, fields, order, limit, show_archived, **kwargs).get_rows()

#############################################################################

def mode_list_projects(client, args):
	pyAMI_atlas.utils.print_table(list_projects(client, **args))

	return 0

#############################################################################
# LIST SUBPROJECTS                                                          #
#############################################################################

def list_subprojects(client, patterns = None, fields = None, order = None, limit = None, show_archived = False, **kwargs):
	'''List ATLAS subprojects.

	Args:
	    :client: the pyAMI client [ pyAMI.client.Client ]
	    :patterns: the list of subproject patterns (glob with %) [ list<str> | None ]
	    :fields: the list of extra fields to be shown if the result [ list<str> | None ]
	    :order: the list of fields for ordering the result [ list<str> | None ]
	    :limit: limit number of results [ int | None ]
	    :show_archived: show archived results [ bool ]

	kwargs:
	    additionnal constraints field_name:field_value (glob with %)

	Returns:
	    an array of python dictionnaries.
	'''

	if not 'read_status' in kwargs:
		kwargs['read_status'] = 'VALID'

	return pyAMI.utils.smart_execute(client, 'subprojects', patterns, fields, order, limit, show_archived, **kwargs).get_rows()

#############################################################################

def mode_list_subprojects(client, args):
	pyAMI_atlas.utils.print_table(list_subprojects(client, **args))

	return 0

#############################################################################
# LIST TYPES                                                                #
#############################################################################

def list_types(client, patterns = None, fields = None, order = None, limit = None, show_archived = False, **kwargs):
	'''List ATLAS types.

	Args:
	    :client: the pyAMI client [ pyAMI.client.Client ]
	    :patterns: the list of type patterns (glob with %) [ list<str> | None ]
	    :fields: the list of extra fields to be shown if the result [ list<str> | None ]
	    :order: the list of fields for ordering the result [ list<str> | None ]
	    :limit: limit number of results [ int | None ]
	    :show_archived: show archived results [ bool ]

	kwargs:
	    additionnal constraints field_name:field_value (glob with %)

	Returns:
	    an array of python dictionnaries.
	'''

	if not 'read_status' in kwargs:
		kwargs['read_status'] = 'VALID'

	return pyAMI.utils.smart_execute(client, 'types', patterns, fields, order, limit, show_archived, **kwargs).get_rows()

#############################################################################

def mode_list_types(client, args):
	pyAMI_atlas.utils.print_table(list_types(client, **args))

	return 0

#############################################################################
# LIST SUBTYPES                                                             #
#############################################################################

def list_subtypes(client, patterns = None, fields = None, order = None, limit = None, show_archived = False, **kwargs):
	'''List ATLAS subtypes.

	Args:
	    :client: the pyAMI client [ pyAMI.client.Client ]
	    :patterns: the list of subtype patterns (glob with %) [ list<str> | None ]
	    :fields: the list of extra fields to be shown if the result [ list<str> | None ]
	    :order: the list of fields for ordering the result [ list<str> | None ]
	    :limit: limit number of results [ int | None ]
	    :show_archived: show archived results [ bool ]

	kwargs:
	    additionnal constraints field_name:field_value (glob with %)

	Returns:
	    an array of python dictionnaries.
	'''

	if not 'read_status' in kwargs:
		kwargs['read_status'] = 'VALID'

	return pyAMI.utils.smart_execute(client, 'subtypes', patterns, fields, order, limit, show_archived, **kwargs).get_rows()

#############################################################################

def mode_list_subtypes(client, args):
	pyAMI_atlas.utils.print_table(list_subtypes(client, **args))

	return 0

#############################################################################
# LIST PRODSTEPS                                                            #
#############################################################################

def list_prodsteps(client, patterns = None, fields = None, order = None, limit = None, show_archived = False, **kwargs):
	'''List ATLAS prodsteps.

	Args:
	    :client: the pyAMI client [ pyAMI.client.Client ]
	    :patterns: the list of prodstep patterns (glob with %) [ list<str> | None ]
	    :fields: the list of extra fields to be shown if the result [ list<str> | None ]
	    :order: the list of fields for ordering the result [ list<str> | None ]
	    :limit: limit number of results [ int | None ]
	    :show_archived: show archived results [ bool ]

	kwargs:
	    additionnal constraints field_name:field_value (glob with %)

	Returns:
	    an array of python dictionnaries.
	'''

	if not 'read_status' in kwargs:
		kwargs['read_status'] = 'VALID'

	return pyAMI.utils.smart_execute(client, 'prodsteps', patterns, fields, order, limit, show_archived, **kwargs).get_rows()

#############################################################################

def mode_list_prodsteps(client, args):
	pyAMI_atlas.utils.print_table(list_prodsteps(client, **args))

	return 0

#############################################################################
# LIST NOMENCLATURE                                                         #
#############################################################################

def list_nomenclature(client, patterns = None, fields = None, order = None, limit = None, show_archived = False, **kwargs):
	'''List ATLAS nomenclature.

	Args:
	    :client: the pyAMI client [ pyAMI.client.Client ]
	    :patterns: the list of nomenclature patterns (glob with %) [ list<str> | None ]
	    :fields: the list of extra fields to be shown if the result [ list<str> | None ]
	    :order: the list of fields for ordering the result [ list<str> | None ]
	    :limit: limit number of results [ int | None ]
	    :show_archived: show archived results [ bool ]

	kwargs:
	    additionnal constraints field_name:field_value (glob with %)

	Returns:
	    an array of python dictionnaries.
	'''

	if not 'read_status' in kwargs:
		kwargs['read_status'] = 'VALID'

	return pyAMI.utils.smart_execute(client, 'nomenclature', patterns, fields, order, limit, show_archived, **kwargs).get_rows()

#############################################################################

def mode_list_nomenclature(client, args):
	pyAMI_atlas.utils.print_table(list_nomenclature(client, **args))

	return 0

#############################################################################
# LIST DATASETS                                                             #
#############################################################################

def list_datasets(client, patterns = None, fields = None, order = None, limit = None, show_archived = False, grl = None, **kwargs):
	'''List ATLAS datasets.

	Args:
	    :client: the pyAMI client [ pyAMI.client.Client ]
	    :patterns: the list of dataset patterns (glob with %) [ list<str> | None ]
	    :fields: the list of extra fields to be shown if the result [ list<str> | None ]
	    :order: the list of fields for ordering the result [ list<str> | None ]
	    :limit: limit number of results [ int | None ]
	    :show_archived: show archived results [ bool ]
	    :grl: the Good Run List (GRL) file name [ str | None ]

	kwargs:
	    additionnal constraints field_name:field_value (glob with %)

	Returns:
	    an array of python dictionnaries.
	'''

	#####################################################################
	# APPLY GRL                                                         #
	#####################################################################

	if grl:

		try:
			import xml.dom.minidom

			doc = xml.dom.minidom.parse(grl)

			kwargs['run_number'] = \
				[node.childNodes[0].data
						for node in doc.getElementsByTagName('Run')]

		except:
			raise pyAMI.exception.Error('could not load GRL `%s`' % grl)

	#####################################################################
	# PATCH PERIODS                                                     #
	#####################################################################

	if 'data_period' in kwargs and kwargs['data_period']:

		tmp = []

		last_char = [
			'0', '1', '2', '3',
			'4', '5', '6', '7',
			'8', '9', '%',
		]

		for data_period in pyAMI.utils.to_array(kwargs['data_period'], sep = ','):

			if data_period:

				if data_period[-1] in last_char:
					tmp.append('%s' % data_period)
				else:
					tmp.append('%s%%' % data_period)

		kwargs['data_period'] = tmp

	#####################################################################
	# PATH PATTERNS                                                     #
	#####################################################################

	patterns = [pattern.rstrip('/') for pattern in pyAMI.utils.to_array(patterns, sep = ',')]

	#####################################################################

	if not 'ami_status' in kwargs:
		kwargs['ami_status'] = 'VALID'

	return pyAMI.utils.smart_execute(client, 'datasets', patterns, fields, order, limit, show_archived, **kwargs).get_rows()

#############################################################################

def mode_list_datasets(client, args):
	pyAMI_atlas.utils.print_table(list_datasets(client, **args))

	return 0

#############################################################################
# LIST DATAPERIODS                                                          #
#############################################################################

def list_dataperiods(client, level, year = None):
	'''List ATLAS data periods.

	Args:
	    :client: the pyAMI client [ pyAMI.client.Client ]
	    :level: the period level (1, 2, 3) [ int ]
	    :year: the year [ int | None ]

	Returns:
	    an array of python dictionnaries.
	'''

	command = [
		'AMIListDataPeriods',
		'-periodLevel="%s"' % level,
		'-createdSince="2009-01-01 00:00:00"',
	]

	if year:
		command.append('-projectName="data%02i%%"' % (year % 100))

	return client.execute(command, format = 'dict_object').get_rows()

#############################################################################

def mode_list_dataperiods(client, args):
	pyAMI_atlas.utils.print_table(list_dataperiods(client, args['level'], args['year']))

	return 0

#############################################################################
# LIST RUNS                                                                 #
#############################################################################

def list_runs(client, year = None, data_periods = None):
	'''List ATLAS runs.

	Args:
	    :client: the pyAMI client [ pyAMI.client.Client ]
	    :year: the year [ int | None ]
	    :data_periods: the list of data periods [ list<str> | None ]

	Returns:
	    an array of python dictionnaries.
	'''

	command = [
		'AMIListRuns',
	]

	if year:
		command.append('-projectName="data%02i%%"' % (year % 100))

	result = []

	if data_periods:
		command.append('')

		for data_period in data_periods:
			command[-1] = '-period="%s"' % data_period

			result.extend(client.execute(command, format = 'dict_object').get_rows())

	else:
		result.extend(client.execute(command, format = 'dict_object').get_rows())

	return result

#############################################################################

def mode_list_runs(client, args):
	#####################################################################
	#                                                                   #
	#####################################################################

	runs = list_runs(client, args['year'], args['data_periods'])

	#####################################################################
	#                                                                   #
	#####################################################################

	if not args['long']:

		for run in sorted([run['runNumber'] for run in runs]):
			print(run)

		return 0

	#####################################################################
	# PRINT TABLE                                                       #
	#####################################################################

	pyAMI_atlas.utils.print_table(runs)

	#####################################################################

	return 0

#############################################################################
# LIST FILES                                                                #
#############################################################################

def list_files(client, dataset, total = None, limit = None):
	'''List dataset files.

	Args:
	    :client: the pyAMI client [ pyAMI.client.Client ]
	    :dataset: the logical dataset name [ str ]
	    :total: produce a grand total [ bool | None]
	    :limit: limit number of results [ int | None ]

	Returns:
	    an array of python dictionnaries.
	'''

	command = [
		'AMIListFiles',
		'-logicalDatasetName="%s"' % dataset,
	]

	if total:
		command.append('-total=yes')
	else:
		command.append('-total=no')

	if limit:

		if isinstance(limit, (list, tuple)):
			command.append('-limit=%i,%i' % (limit[0], limit[1]))
		else:
			command.append('-limit=0,%i' % limit)

	return client.execute(command, format = 'dict_object').get_rows()

#############################################################################

def mode_list_files(client, args):
	#####################################################################
	#                                                                   #
	#####################################################################

	datasets = list_files(client, args['dataset'], total = args['count'], limit = args['limit'])

	#####################################################################
	#                                                                   #
	#####################################################################

	if not args['long']:

		for dataset in datasets:
			print(dataset['LFN'])

		return 0

	#####################################################################
	#                                                                   #
	#####################################################################

	if args['human']:

		for dataset in datasets:
			dataset['events'] = pyAMI_atlas.utils.humanize_numbs(dataset['events'])
			dataset['fileSize'] = pyAMI_atlas.utils.humanize_bytes(dataset['fileSize'])

	#####################################################################
	# PRINT TABLE                                                       #
	#####################################################################

	pyAMI_atlas.utils.print_table(datasets)

	#####################################################################

	return 0

#############################################################################
